//////////////////////////////////////////////////////////////////////
//  JJoyStick2Obj.cpp - Joy stick external module for NeatTools
//						Coded by Mike Fusco
//						Tue Feb  4 14:32:24 PST 2003
//
//						Based off of JJoyStickObj.cpp by Yuh-Jye Chang
//                      and Rob Salgado's ModuleMaker
//
//////////////////////////////////////////////////////////////////////

#include "JJoyStick2Obj.h"
#include "JLinkObj.h"
///
#include "JInteger.h"
#include "JIntegerData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
///
//#include "JRealData.h"
//#include "JRealProperty.h"
///
#include "JString.h"
#include "JStringData.h"
#include "JStringProperty.h"
//
//#include "JBytesData.h" 
//
#include <stdio.h>
#include <malloc.h>
#include <string.h>
//#include <math.h>

#if defined(WIN32)
  #include <windows.h>
#endif

char* joyTag2[] = {"Joy1", "Joy2", null};
char* joyVersion[] = {"1.1", null};

// ------------------------------------------------------------------------ //
//
//  Register() registers the module with NeatTools
//
char* theJJoyStick2Obj = JJoyStick2Obj().Register();

// ------------------------------------------------------------------------ //
//
void JJoyStick2Obj::writeContent(JOutputStream& os) {
    JAddObj::writeContent(os);  
  
	putInteger(os, "id", id);
}

// ------------------------------------------------------------------------ //
//
void JJoyStick2Obj::readContent(JDictionary& dict) {
    JAddObj::readContent(dict); 
  
	id = getInteger(dict, "id");
	valid = isValid();

}

// ------------------------------------------------------------------------ //
//
//  className() returns the name of the module
//
const char* JJoyStick2Obj::className() const { return "JJoyStick2Obj";}

// ------------------------------------------------------------------------ //
//
//  clone() is used to instantiate copies of the module
//
JObject* JJoyStick2Obj::clone() const { return new JJoyStick2Obj(*this);}

// ------------------------------------------------------------------------ //
//
//  Constructor method
//
JJoyStick2Obj::JJoyStick2Obj(int _id ) {

	id = _id;

//	valid = isValid();

	valid=true;

    igm = NUMBER_OF_INPUTS;     // tell NeatTools the number of inputs (declared in JModuleObj)
    ogm = NUMBER_OF_OUTPUTS;    // tell NeatTools the number of outputs (declared in JModuleObj)

    doRepaint = false;

    moduleColor=JColor::green.brighter();   //see JColor.h for definitions

	for(int i=0;i<NUMBER_OF_OUTPUTS;i++) outPutChannels[i]=0;
	clock=0;
}

boolean JJoyStick2Obj::isValid() {
#if defined(WIN32)
	JOYINFO ji;
	if (JOYERR_NOERROR == joyGetPos(id, &ji)) return true;
	return false;
#else
  return false;
#endif
}


///void JJoyStick2Obj::startup(void) {}
///void JJoyStick2Obj::reset(void) {}
///void JJoyStick2Obj::close(void) {}


// ------------------------------------------------------------------------ //
//
//  Copy Constructor method
//
/// JJoyStick2Obj::JJoyStick2Obj(const JJoyStick2Obj& src):JAddObj(*(JAddObj*)& src) 
/// {
///          use your own copy constructor
///          and NOT the compiler-generated one
///          to do memberwise-copy of arrays
/// } 

// ------------------------------------------------------------------------ //
//
//  paint() method is called by the system
//
void JJoyStick2Obj::paint(JGraphics g, double dx, double dy, 
                                JRegion& rgn, double scale) {
	//determine the size or extent of the module
	JRect rect = getIExtent(dx, dy, scale);
 
	//the next "write" to screenbuffer g is the appropriate color
	g.setJColor(moduleColor);

	//draw a box with a 3D-look that fills the module extent
	g.fill3DRect(rect.x, rect.y, rect.width, rect.height, 3);

	valid = isValid();
	g.setJColor(JColor::black.brighter());
	if (!valid) g.setJColor(JColor::gray);
	drawText(g,joyTag2[id],rect);



}

// ------------------------------------------------------------------------ //
//
//
int JJoyStick2Obj::inputType(int n) { 

	return JIntegerData::id;

}

int JJoyStick2Obj::outputType(int n) { 

	return JIntegerData::id;

}


boolean JJoyStick2Obj::inputAllowed(int n){ 
    return !inputSet(n).last(); //accept only one link per input port
}
// ------------------------------------------------------------------------ //
boolean JJoyStick2Obj::outputAllowed(int n){ 
    return true;        //accept as many links as needed
}

// ------------------------------------------------------------------------ //
JString JJoyStick2Obj::inputTag(int n) {
    char* tag[] = {"clock"};    //label inputs

    return tag[0];
}
// ------------------------------------------------------------------------ //
JString JJoyStick2Obj::outputTag(int n) {    
    char* tag[] = {
					"num buttons down",
					"button 1",
					"button 2",
					"button 3",
					"button 4",
					"button 5",
					"button 6",
					"button 7",
					"button 8",
					"button 9",
					"button 10",
					"button 11",
					"button 12",
					"button 13",
					"button 14",
					"button 15",
					"button 16",
					"button 17",
					"button 18",
					"button 19",
					"button 20",
					"button 21",
					"button 22",
					"button 23",
					"button 24",
					"button 25",
					"button 26",
					"button 27",
					"button 28",
					"button 29",
					"button 30",
					"button 31",
					"button 32",
					"POV angle deg * 100",
					"joy 1 x",
					"joy 1 y",
					"joy 2 x",
					"joy 2 y",
					"joy 3 x",
					"joy 3 y",
					"ERROR"
	};

    if ( n < NUMBER_OF_OUTPUTS && n >= 0) return tag[n];
	return tag[NUMBER_OF_OUTPUTS];
}
// ------------------------------------------------------------------------ //


// ------------------------------------------------------------------------ //
int JJoyStick2Obj::inputFace(int n) { 
    return TOP;        //all inputs on the topFace
}
// ------------------------------------------------------------------------ //
int JJoyStick2Obj::outputFace(int n) { 

	//if (n<OUT_POV) return BOTTOM;
	return RIGHT;
}

// ------------------------------------------------------------------------ //
//  inputArea, outputArea   defines the area that a port will accept connections
//      leftArea(int portNumber,
//              int whichPortSlotInThisArea,
//              int howManyPortSlotsInThisArea,
//              double whereToStartAreaFraction (default 0.25),
//              double widthAreaFraction (default is 0.50)
//              double perpendicularReciprocalFactor (default is 6)     )
//ex:
//      return leftArea(n, 0, 1);           //port-0 is alone on LEFT
//

JFRect JJoyStick2Obj::inputArea(int n) {
    return topArea(n, 0, 1);   //one inputs on the topArea, starting with 0
}
// ------------------------------------------------------------------------ //
JFRect JJoyStick2Obj::outputArea(int n) {

//	if (n<OUT_POV) return bottomArea(n, 0, OUT_POV);
//	return rightArea(n, OUT_POV, NUMBER_OF_OUTPUTS-OUT_POV);
	return rightArea(n, 0, NUMBER_OF_OUTPUTS,.1,0.8);
}

// ------------------------------------------------------------------------ //
// for fine-tuned positioning (for possibly multiple links) with the areas

JFPoint JJoyStick2Obj::inputPoint(int n, JLinkObj& link) { 
    return topPoint(n, link, 0, 1);    
}
// ------------------------------------------------------------------------ //
JFPoint JJoyStick2Obj::outputPoint(int n, JLinkObj& link) { 

//	if (n<OUT_POV) return bottomPoint(n, link, 0, OUT_POV);
//	return rightPoint(n, link, OUT_POV, NUMBER_OF_OUTPUTS-OUT_POV);
	return rightPoint(n, link, 0, NUMBER_OF_OUTPUTS,.1,0.8);
}



// ------------------------------------------------------------------------ //
//
// engine() is called when an input-event is detected
//
void JJoyStick2Obj::engine(int n, JLinkObj& link) {

  if (!valid) return;

#if defined(WIN32)

  int oldClock = clock;
  link.access(JIntegerData(clock));
  clock = (clock != 0);
  if ((clock != oldClock) && clock) {

		UINT uJoyID = id;
		JOYINFOEX pji;
		pji.dwSize = sizeof(JOYINFOEX);
		pji.dwFlags = JOY_RETURNALL;
		joyGetPosEx(uJoyID,&pji);
		
		outPutChannels[OUT_BUTTONNUMBER] = pji.dwButtonNumber;
		
		DWORD bmask = JOY_BUTTON1;
		DWORD buttonV = pji.dwButtons;
		
		for(int i=OUT_FIRSTBT;i<OUT_POV;i++) {
			outPutChannels[i] = ((buttonV & bmask) != 0) ? bmask : 0;
			bmask = bmask<<1;
			//outPutChannels[i] = bmask;
		}
		
		outPutChannels[OUT_POV] = pji.dwPOV;
		
		outPutChannels[OUT_JOY  ] = pji.dwXpos;
		outPutChannels[OUT_JOY+1] = pji.dwYpos;
		outPutChannels[OUT_JOY+2] = pji.dwZpos;
		outPutChannels[OUT_JOY+3] = pji.dwRpos;
		outPutChannels[OUT_JOY+4] = pji.dwUpos;
		outPutChannels[OUT_JOY+5] = pji.dwVpos;

		for( i=OUT_BUTTONNUMBER;i<NUMBER_OF_OUTPUTS;i++) 
			broadcast(i); //ping all outputs
    }
#else
	return;
#endif
}

// ------------------------------------------------------------------------ //
//
// access() may be called by an output-link engine() method 
//      when we do a broadcast()
//
void JJoyStick2Obj::access(int n, JLinkObj& link, const JDataType& data) {

	INT(data)= outPutChannels[n];

}
        

// ------------------------------------------------------------------------ //
//
// getProperties() produces a list of properties
//
JArray JJoyStick2Obj::getProperties() {
  JArray properties = JAddObj::getProperties();
  properties.append(JIntegerListProperty("port", id, joyTag2));
  properties.append(JIntegerListProperty("version", 0, joyVersion));
  return properties;
}

// ------------------------------------------------------------------------ //
//
// updateProperty() updates a given property, as specified by VARIABLE_LABEL
//

boolean JJoyStick2Obj::updateProperty(JProperty& prop) {

  if (JAddObj::updateProperty(prop)) return true;

  if (prop.getName() == JString("port")) {
    id = ((JIntegerListProperty*)&prop)->value;
    valid = isValid();
    repaint();
    return true;
  }
  return false;
}

// ------------------------------------------------------------------------ //

void JJoyStick2Obj::addButton(JComponent* panel) { 
  addModuleButton(panel, JJoyStick2Obj(0));
  addModuleButton(panel, JJoyStick2Obj(1));
}


